/*
 * Copyright 1999-2018 Alibaba Group Holding Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alibaba.csp.sentinel.dashboard.datasource.entity;

import com.chushang.common.mongo.annotation.MongoDel;
import org.springframework.data.mongodb.core.mapping.Document;
import org.springframework.data.mongodb.core.mapping.MongoId;

import java.io.Serializable;

/**
 * @author leyou
 */
@Document(value = "MetricEntity")
@MongoDel
public class MetricEntity implements Serializable {
	private static final long serialVersionUID = 1L;

	@MongoId
	private String mongoId;

	private Long id;
	/**
	 * 时间戳
	 */
	private Long gmtCreate;

	private Long gmtModified;
	/**
	 * 应用名称
	 */
	private String app;

	/**
	 * 监控信息的时间戳
	 */
	private Long timestamp;
	/**
	 * 资源名称
	 */
	private String resource;
	/**
	 * 通过qps
	 */
	private Long passQps;
	/**
	 * 成功qps
	 */
	private Long successQps;
	/**
	 * 限流qps
	 */
	private Long blockQps;
	/**
	 * 异常qps
	 */
	private Long exceptionQps;

	/**
	 * summary rt of all success exit qps.
	 */
	private double rt;

	/**
	 * 本次聚合的总条数
	 */
	private int count;

	private int resourceCode;

	public static MetricEntity copyOf(MetricEntity oldEntity) {
		MetricEntity entity = new MetricEntity();
		entity.setId(oldEntity.getId());
		entity.setGmtCreate(oldEntity.getGmtCreate());
		entity.setGmtModified(oldEntity.getGmtModified());
		entity.setApp(oldEntity.getApp());
		entity.setTimestamp(oldEntity.getTimestamp());
		entity.setResource(oldEntity.getResource());
		entity.setPassQps(oldEntity.getPassQps());
		entity.setBlockQps(oldEntity.getBlockQps());
		entity.setSuccessQps(oldEntity.getSuccessQps());
		entity.setExceptionQps(oldEntity.getExceptionQps());
		entity.setRt(oldEntity.getRt());
		entity.setCount(oldEntity.getCount());
		return entity;
	}

	public synchronized void addPassQps(Long passQps) {
		this.passQps += passQps;
	}

	public synchronized void addBlockQps(Long blockQps) {
		this.blockQps += blockQps;
	}

	public synchronized void addExceptionQps(Long exceptionQps) {
		this.exceptionQps += exceptionQps;
	}

	public synchronized void addCount(int count) {
		this.count += count;
	}

	public synchronized void addRtAndSuccessQps(double avgRt, Long successQps) {
		this.rt += avgRt * successQps;
		this.successQps += successQps;
	}

	/**
	 * {@link #rt} = {@code avgRt * successQps}
	 * @param avgRt average rt of {@code successQps}
	 * @param successQps
	 */
	public synchronized void setRtAndSuccessQps(double avgRt, Long successQps) {
		this.rt = avgRt * successQps;
		this.successQps = successQps;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getGmtCreate() {
		return gmtCreate;
	}

	public void setGmtCreate(Long gmtCreate) {
		this.gmtCreate = gmtCreate;
	}

	public Long getGmtModified() {
		return gmtModified;
	}

	public void setGmtModified(Long gmtModified) {
		this.gmtModified = gmtModified;
	}

	public String getApp() {
		return app;
	}

	public void setApp(String app) {
		this.app = app;
	}

	public Long getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Long timestamp) {
		this.timestamp = timestamp;
	}

	public String getResource() {
		return resource;
	}

	public void setResource(String resource) {
		this.resource = resource;
		this.resourceCode = resource.hashCode();
	}

	public Long getPassQps() {
		return passQps;
	}

	public void setPassQps(Long passQps) {
		this.passQps = passQps;
	}

	public Long getBlockQps() {
		return blockQps;
	}

	public void setBlockQps(Long blockQps) {
		this.blockQps = blockQps;
	}

	public Long getExceptionQps() {
		return exceptionQps;
	}

	public void setExceptionQps(Long exceptionQps) {
		this.exceptionQps = exceptionQps;
	}

	public double getRt() {
		return rt;
	}

	public void setRt(double rt) {
		this.rt = rt;
	}

	public int getCount() {
		return count;
	}

	public void setCount(int count) {
		this.count = count;
	}

	public int getResourceCode() {
		return resourceCode;
	}

	public Long getSuccessQps() {
		return successQps;
	}

	public void setSuccessQps(Long successQps) {
		this.successQps = successQps;
	}


	public String getMongoId() {
		return mongoId;
	}

	public void setMongoId(String mongoId) {
		this.mongoId = mongoId;
	}
}
